"""
ldapcontrol.py - helper class to define ldap-controls
written by Ingo Steuwer, <steuwer@univention.de>

See http://python-ldap.sourceforge.net for details.
"""

from ldap import LDAPError

__all__ = [
  'LDAPControl',
]


# Well know OID's:

# Active Directory

LDAP_PAGED_RESULT_OID_STRING         = "1.2.840.113556.1.4.319"
LDAP_SERVER_ASQ_OID                  = "1.2.840.113556.1.4.1504"
LDAP_SERVER_CROSSDOM_MOVE_TARGET_OID = "1.2.840.113556.1.4.521"
LDAP_SERVER_DIRSYNC_OID              = "1.2.840.113556.1.4.841"
LDAP_SERVER_DOMAIN_SCOPE_OID         = "1.2.840.113556.1.4.1339"
LDAP_SERVER_EXTENDED_DN_OID          = "1.2.840.113556.1.4.529" # no berElement needed
LDAP_SERVER_LAZY_COMMIT_OID          = "1.2.840.113556.1.4.619" # no berElement needed
LDAP_SERVER_NOTIFICATION_OID         = "1.2.840.113556.1.4.528" # no berElement needed
LDAP_SERVER_PERMISSIVE_MODIFY_OID    = "1.2.840.113556.1.4.1413" # no berElement needed
LDAP_SERVER_QUOTA_CONTROL_OID        = "1.2.840.113556.1.4.1852"
LDAP_SERVER_RESP_SORT_OID            = "1.2.840.113556.1.4.474"
LDAP_SERVER_SD_FLAGS_OID             = "1.2.840.113556.1.4.801"
LDAP_SERVER_SEARCH_OPTIONS_OID       = "1.2.840.113556.1.4.1340"
LDAP_SERVER_SHOW_DELETED_OID         = "1.2.840.113556.1.4.417" # no berElement needed
LDAP_SERVER_SORT_OID                 = "1.2.840.113556.1.4.473"
LDAP_SERVER_TREE_DELETE_OID          = "1.2.840.113556.1.4.805" # no berElement needed
LDAP_SERVER_VERIFY_NAME_OID          = "1.2.840.113556.1.4.1338"
LDAP_CONTROL_VLVREQUEST              = "2.16.840.1.113730.3.4.9"
LDAP_CONTROL_VLVRESPONSE             = "2.16.840.1.113730.3.4.10"

# openldap (2.1.30)

# from ldap.h
LDAP_CONTROL_VALUESRETURNFILTER      = "1.2.826.0.1.334810.2.3"
LDAP_CONTROL_SUBENTRIES              = "1.3.6.1.4.1.4203.1.10.1"
LDAP_CONTROL_NOOP                    = "1.3.6.1.4.1.4203.1.10.2"
LDAP_CONTROL_MANAGEDSAIT             = "2.16.840.1.113730.3.4.2"
LDAP_CONTROL_PROXY_AUTHZ             = "2.16.840.1.113730.3.4.18"

# these values are defined but not supported - and also available
# by Active Directory (some with different names)
LDAP_CONTROL_SORTREQUEST             = "1.2.840.113556.1.4.473" # defined not supported
LDAP_CONTROL_SORTRESPONSE            = "1.2.840.113556.1.4.474" # defined not supported
#LDAP_CONTROL_VLVREQUEST              = "2.16.840.1.113730.3.4.9" # defined not supported
#LDAP_CONTROL_VLVRESPONSE             = "2.16.840.1.113730.3.4.10" # defined not supported


class LDAPControl:
	"""
	Helper-Class for defining LDAP-Controls
	"""
	
	def __init__(self, oid, isCritical = 1, berElement = None):
		
		self.oid = oid
		self.isCritical = int(isCritical)
		self.berElement = berElement
		
	def __repr__(self):
		return self.as_tupel().__repr__()
		
	def as_tupel(self):
		return (self.oid, self.isCritical, self.berElement)
		
	def from_tupel(self, list):
		
		if type(list) != list or len(list) != 3 :
			raise VauleError, 'expected list with length 3 to create LDAPControl'
		
		self.oid = list[0]
		self.isCritical = list[1]
		self.berElement = list[2]
