#include "Python.h"

#include "ldapcontrol.h"

#include "lber.h"
#include "ldap.h"


LDAPControl* PyControl_to_LDAPControl( PyObject* pyControl )
{
  char* oid;
  int isCritical;
  char* method = "as_tupel";

  PyObject *pyBerElement = Py_None;
  PyObject *ctrlTupel = Py_None;

  BerElement *pbe;

  LDAPControl *ctrlp;
  
  if (!PyObject_HasAttrString( pyControl, method ))
	{
	  PyErr_SetObject(PyExc_TypeError, Py_BuildValue("sO",
	   "expected a LDAPControl", pyControl));
	  return NULL;
	}

  ctrlTupel = PyObject_CallMethod ( pyControl, method, NULL );
  if (!ctrlTupel)
	{
	  printf("parsetuple failed\n");
	  PyErr_SetObject(PyExc_TypeError, Py_BuildValue("sO",
	   "expected a tuple (returned from method \"as_tupel\")", ctrlTupel));
	  return NULL;
	}  

  if (!PyArg_ParseTuple( ctrlTupel, "si|O",
						 &oid, &isCritical, &pyBerElement )) 
	{
	  PyErr_SetObject(PyExc_TypeError, Py_BuildValue("sO",
	   "expected a tupel with 3 elements (returned from method \"as_tupel\")", ctrlTupel));
	  return NULL;
	}
  
  // TODO: something like "pbe = PyBerElement_to_BerElement( pyBerElement )"
  pbe = ber_alloc_t(LBER_USE_DER);

  if ( pbe == NULL ) {
	return NULL;
  }

  ldap_create_control(oid, pbe, isCritical, &ctrlp);

  return ctrlp;
}


//LDAPControl** PyControls_to_LDAPControls( PyObject* pyControls )
int PyControls_to_LDAPControls( PyObject* pyControls, LDAPControl ***sctrls )
{
  if (!PySequence_Check( pyControls ))
	{
	  PyErr_SetObject(PyExc_TypeError, Py_BuildValue("sO",
	   "expected a sequence", pyControls));
	  return -2;
	}

  int len = PySequence_Length( pyControls );

  if ( len < 0 ) {
	PyErr_SetObject(PyExc_TypeError, Py_BuildValue("sO",
     "sequence has unknown length", pyControls));
	return -1;
  }

  LDAPControl **ctrls;

  ctrls = malloc((len+1)*sizeof(LDAPControl));

  PyObject* pyControl;
  int i;
  
  for ( i=0; i<len; i++)
	{
	  pyControl = PySequence_GetItem( pyControls, i );
	  ctrls[i] = PyControl_to_LDAPControl( pyControl );
	}
  ctrls[len] = NULL;
    
  *sctrls = ctrls;

  return 0;
}
